﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Net;
using System.IO;
using System.Globalization;
using System.Drawing;

namespace GUI_pristani
{
    class Moon
    {
        public int ID { get; private set; }
        public float X { get; private set; }
        public float Y { get; private set; }
        public float Vx { get; private set; }
        public float Vy { get; private set; }
        public float P { get; private set; }
        public float maxP { get; private set; }
        public float L { get; private set; }
        public float R { get; private set; }
        public string uri { get; private set; }
        private Point offset { get; set; }
        private float scale { get; set; }
        public bool up { get; private set; }
        public bool left { get; private set; }
        public bool right { get; private set; }

        public int state { get; private set; }

        public Moon(string uri) {
            offset = new Point(0, 0);
            scale = 3;

            if (!uri.EndsWith("/")) uri += "/";
            this.uri = uri;

            string[] result = request("start");

            ID = int.Parse(result[0]);

            string[] line = result[1].Split(' ');
            X = float.Parse(line[0], CultureInfo.InvariantCulture);
            Y = float.Parse(line[1], CultureInfo.InvariantCulture);

            line = result[2].Split(' ');
            Vx = float.Parse(line[0], CultureInfo.InvariantCulture);
            Vy = float.Parse(line[1], CultureInfo.InvariantCulture);

            P = float.Parse(result[3], CultureInfo.InvariantCulture);
            maxP = P;

            line = result[4].Split(' ');
            L = float.Parse(line[0], CultureInfo.InvariantCulture);
            R = float.Parse(line[1], CultureInfo.InvariantCulture);
        }

        public void refresh(bool up, bool left, bool right)
        {
            this.up = up;
            this.left = left;
            this.right = right;

            string[] result = request("stav?id=" + ID + "&nahoru=" + (up ? "1" : "0") + "&doleva=" + (left ? "1" : "0") + "&doprava=" + (right ? "1" : "0"));

            if (result.Count() < 4)
            {
                if (result[0].StartsWith("Zdar"))
                {
                    state = 1;
                }
                if (result[0].StartsWith("Krach"))
                {
                    state = 2;
                }
            }
            else
            {

                ID = int.Parse(result[0]);

                string[] line = result[1].Split(' ');
                X = float.Parse(line[0], CultureInfo.InvariantCulture);
                Y = float.Parse(line[1], CultureInfo.InvariantCulture);

                line = result[2].Split(' ');
                Vx = float.Parse(line[0], CultureInfo.InvariantCulture);
                Vy = float.Parse(line[1], CultureInfo.InvariantCulture);

                P = float.Parse(result[3], CultureInfo.InvariantCulture);

                line = result[4].Split(' ');
                L = float.Parse(line[0], CultureInfo.InvariantCulture);
                R = float.Parse(line[1], CultureInfo.InvariantCulture);
            }
        }

        private string[] request(string page) {
            List<string> html = new List<string>();
            string url = uri + page;

            HttpWebRequest request = (HttpWebRequest)WebRequest.Create(url);
            request.AutomaticDecompression = DecompressionMethods.GZip;

            using (HttpWebResponse response = (HttpWebResponse)request.GetResponse())
            using (Stream stream = response.GetResponseStream())
            using (StreamReader reader = new StreamReader(stream))
            {
                while (!reader.EndOfStream) {
                    html.Add(reader.ReadLine());
                }
            }

            return html.ToArray();
        }

        public void paint(Graphics g, int height, int width) {
            Color surface = Color.Gray;
            Color plate = Color.White;
            Image image = GUI_pristani.Properties.Resources.image;
            Pen surfacePen = new Pen(surface, 2);
            surfacePen.DashPattern = new float[] { 1, 1 };
            Pen platePen = new Pen(plate, 2);

            g.DrawLine(surfacePen, scaledPoint(new PointF(-1000, 0), height), scaledPoint(new PointF(L, 0), height));
            g.DrawLine(surfacePen, scaledPoint(new PointF(1000, 0), height), scaledPoint(new PointF(R, 0), height));
            g.DrawLine(platePen, scaledPoint(new PointF(L, 0), height), scaledPoint(new PointF(R, 0), height));

            g.DrawImage(image, new RectangleF(scaledPointMoved(new PointF(X, Y), height, new PointF(-10, -10)), new SizeF(20 * scale, 20 * scale)));
            if (right) { g.FillRectangle(new SolidBrush(Color.Red), new RectangleF(scaledPointMoved(new PointF(X, Y), height, new PointF(-15, 3)), new SizeF(8 * scale, 3 * scale))); }
            if (left) { g.FillRectangle(new SolidBrush(Color.Red), new RectangleF(scaledPointMoved(new PointF(X, Y), height, new PointF(7, 3)), new SizeF(8 * scale, 3 * scale))); }
            if (up) { g.FillRectangle(new SolidBrush(Color.Red), new RectangleF(scaledPointMoved(new PointF(X, Y), height, new PointF((float)-2.5, 8)), new SizeF(5 * scale, 8 * scale))); }

            g.FillRectangle(new SolidBrush(Color.Black), width - 60, 0, 60, height);

            g.FillRectangle(new SolidBrush(Color.FromArgb((int)(255 - (P/ maxP) *255), (int)(0 + (P / maxP) * 150), 0)), width - 50, (float)(height - 10 - (height - 20)*(P/ maxP)) , 40, (float)((height - 20) * (P / maxP)));
            g.DrawRectangle(new Pen(Color.White, 2), width - 50, 10, 40, height - 20);

            g.DrawString("Rychlost: Y=" + Vy.ToString("0.00") + " X=" + Vy.ToString("0.00"), new Font(new FontFamily("Courier New"), 10, FontStyle.Regular, GraphicsUnit.Pixel), new SolidBrush(Color.White), 10, height - 20);
        }

        private PointF scaledPoint(PointF p, int height) {
            p.X += offset.X;
            p.X *= scale;

            p.Y += offset.X;
            p.Y *= scale;
            p.Y = height - p.Y - 30;

            return p;
        }

        private PointF scaledPointMoved(PointF p, int height, PointF move) {
            p = scaledPoint(p, height);
            p.X += move.X * scale;
            p.Y += move.Y * scale;

            return p;
        }
    }
}
